@@ -33,17 +33,16 @@ import java.util.Random;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
-import java.util.function.Consumer;
 
 import org.apache.hc.client5.http.HttpRequestRetryStrategy;
 import org.apache.hc.client5.http.classic.methods.HttpGet;
 import org.apache.hc.client5.http.classic.methods.HttpPost;
-import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
-import org.apache.hc.client5.http.impl.classic.HttpClientBuilder;
+import org.apache.hc.client5.http.impl.io.PoolingHttpClientConnectionManager;
 import org.apache.hc.client5.http.protocol.HttpClientContext;
 import org.apache.hc.client5.http.protocol.RedirectLocations;
 import org.apache.hc.client5.http.utils.URIUtils;
-import org.apache.hc.client5.testing.sync.extension.TestClientResources;
+import org.apache.hc.client5.testing.extension.sync.ClientProtocolLevel;
+import org.apache.hc.client5.testing.extension.sync.TestClient;
 import org.apache.hc.core5.http.ClassicHttpRequest;
 import org.apache.hc.core5.http.ClassicHttpResponse;
 import org.apache.hc.core5.http.Header;
@@ -57,52 +56,25 @@ import org.apache.hc.core5.http.URIScheme;
 import org.apache.hc.core5.http.impl.io.HttpRequestExecutor;
 import org.apache.hc.core5.http.io.HttpClientConnection;
 import org.apache.hc.core5.http.io.HttpRequestHandler;
+import org.apache.hc.core5.http.io.HttpResponseInformationCallback;
 import org.apache.hc.core5.http.io.entity.EntityUtils;
 import org.apache.hc.core5.http.io.entity.InputStreamEntity;
 import org.apache.hc.core5.http.io.entity.StringEntity;
 import org.apache.hc.core5.http.message.BasicClassicHttpRequest;
 import org.apache.hc.core5.http.protocol.HttpContext;
 import org.apache.hc.core5.net.URIBuilder;
-import org.apache.hc.core5.testing.classic.ClassicTestServer;
 import org.apache.hc.core5.util.TimeValue;
-import org.apache.hc.core5.util.Timeout;
 import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.Disabled;
 import org.junit.jupiter.api.Test;
-import org.junit.jupiter.api.extension.RegisterExtension;
 
 /**
  * Client protocol handling tests.
  */
-public abstract class TestClientRequestExecution {
+abstract  class TestClientRequestExecution extends AbstractIntegrationTestBase {
 
-    public static final Timeout TIMEOUT = Timeout.ofMinutes(1);
-
-    @RegisterExtension
-    private TestClientResources testResources;
-
-    protected TestClientRequestExecution(final URIScheme scheme) {
-        this.testResources = new TestClientResources(scheme, TIMEOUT);
-    }
-
-    public URIScheme scheme() {
-        return testResources.scheme();
-    }
-
-    public ClassicTestServer startServer() throws IOException {
-        return testResources.startServer(null, null, null);
-    }
-
-    public CloseableHttpClient startClient(final Consumer<HttpClientBuilder> clientCustomizer) throws Exception {
-        return testResources.startClient(clientCustomizer);
-    }
-
-    public CloseableHttpClient startClient() throws Exception {
-        return testResources.startClient(builder -> {});
-    }
-
-    public HttpHost targetHost() {
-        return testResources.targetHost();
+    public TestClientRequestExecution(final URIScheme scheme) {
+        super(scheme, ClientProtocolLevel.STANDARD);
     }
 
     private static class SimpleService implements HttpRequestHandler {
@@ -115,7 +87,7 @@ public abstract class TestClientRequestExecution {
         public void handle(
                 final ClassicHttpRequest request,
                 final ClassicHttpResponse response,
-                final HttpContext context) throws HttpException, IOException {
+                final HttpContext context) {
             response.setCode(HttpStatus.SC_OK);
             final StringEntity entity = new StringEntity("Whatever");
             response.setEntity(entity);
@@ -137,8 +109,17 @@ public abstract class TestClientRequestExecution {
                 final ClassicHttpRequest request,
                 final HttpClientConnection conn,
                 final HttpContext context) throws IOException, HttpException {
+            return execute(request, conn, null, context);
+        }
 
-            final ClassicHttpResponse response = super.execute(request, conn, context);
+        @Override
+        public ClassicHttpResponse execute(
+                final ClassicHttpRequest request,
+                final HttpClientConnection conn,
+                final HttpResponseInformationCallback informationCallback,
+                final HttpContext context) throws IOException, HttpException {
+
+            final ClassicHttpResponse response = super.execute(request, conn, informationCallback, context);
             final Object marker = context.getAttribute(MARKER);
             if (marker == null) {
                 context.setAttribute(MARKER, Boolean.TRUE);
@@ -150,10 +131,9 @@ public abstract class TestClientRequestExecution {
     }
 
     @Test
-    public void testAutoGeneratedHeaders() throws Exception {
-        final ClassicTestServer server = startServer();
-        server.registerHandler("*", new SimpleService());
-        final HttpHost target = targetHost();
+    void testAutoGeneratedHeaders() throws Exception {
+        configureServer(bootstrap -> bootstrap.register("*", new SimpleService()));
+        final HttpHost target = startServer();
 
         final HttpRequestInterceptor interceptor = (request, entityDetails, context) -> request.addHeader("my-header", "stuff");
 
@@ -186,11 +166,12 @@ public abstract class TestClientRequestExecution {
 
         };
 
-        final CloseableHttpClient client = startClient(builder -> builder
+        configureClient(builder -> builder
                 .addRequestInterceptorFirst(interceptor)
                 .setRequestExecutor(new FaultyHttpRequestExecutor("Oppsie"))
                 .setRetryStrategy(requestRetryStrategy)
         );
+        final TestClient client = client();
 
         final HttpClientContext context = HttpClientContext.create();
 
@@ -210,10 +191,9 @@ public abstract class TestClientRequestExecution {
     }
 
     @Test
-    public void testNonRepeatableEntity() throws Exception {
-        final ClassicTestServer server = startServer();
-        server.registerHandler("*", new SimpleService());
-        final HttpHost target = targetHost();
+    void testNonRepeatableEntity() throws Exception {
+        configureServer(bootstrap -> bootstrap.register("*", new SimpleService()));
+        final HttpHost target = startServer();
 
         final HttpRequestRetryStrategy requestRetryStrategy = new HttpRequestRetryStrategy() {
 
@@ -244,10 +224,11 @@ public abstract class TestClientRequestExecution {
 
         };
 
-        final CloseableHttpClient client = startClient(builder -> builder
+        configureClient(builder -> builder
                 .setRequestExecutor(new FaultyHttpRequestExecutor("a message showing that this failed"))
                 .setRetryStrategy(requestRetryStrategy)
         );
+        final TestClient client = client();
 
         final HttpClientContext context = HttpClientContext.create();
 
@@ -261,12 +242,11 @@ public abstract class TestClientRequestExecution {
     }
 
     @Test
-    public void testNonCompliantURI() throws Exception {
-        final ClassicTestServer server = startServer();
-        server.registerHandler("*", new SimpleService());
-        final HttpHost target = targetHost();
+    void testNonCompliantURI() throws Exception {
+        configureServer(bootstrap -> bootstrap.register("*", new SimpleService()));
+        final HttpHost target = startServer();
 
-        final CloseableHttpClient client = startClient();
+        final TestClient client = client();
 
         final HttpClientContext context = HttpClientContext.create();
         final ClassicHttpRequest request = new BasicClassicHttpRequest("GET", "{{|boom|}}");
@@ -282,12 +262,11 @@ public abstract class TestClientRequestExecution {
     }
 
     @Test
-    public void testRelativeRequestURIWithFragment() throws Exception {
-        final ClassicTestServer server = startServer();
-        server.registerHandler("*", new SimpleService());
-        final HttpHost target = targetHost();
+    void testRelativeRequestURIWithFragment() throws Exception {
+        configureServer(bootstrap -> bootstrap.register("*", new SimpleService()));
+        final HttpHost target = startServer();
 
-        final CloseableHttpClient client = startClient();
+        final TestClient client = client();
 
         final HttpGet httpget = new HttpGet("/stuff#blahblah");
         final HttpClientContext context = HttpClientContext.create();
@@ -303,12 +282,11 @@ public abstract class TestClientRequestExecution {
     }
 
     @Test
-    public void testAbsoluteRequestURIWithFragment() throws Exception {
-        final ClassicTestServer server = startServer();
-        server.registerHandler("*", new SimpleService());
-        final HttpHost target = targetHost();
+    void testAbsoluteRequestURIWithFragment() throws Exception {
+        configureServer(bootstrap -> bootstrap.register("*", new SimpleService()));
+        final HttpHost target = startServer();
 
-        final CloseableHttpClient client = startClient();
+        final TestClient client = client();
 
         final URI uri = new URIBuilder()
             .setHost(target.getHostName())
@@ -337,13 +315,12 @@ public abstract class TestClientRequestExecution {
     @Test @Disabled("Fails intermittently with GitHub Actions")
     public void testRequestCancellation() throws Exception {
         startServer();
-        final HttpHost target = targetHost();
+        final HttpHost target = startServer();
 
-        final CloseableHttpClient client = testResources.startClient(
-                builder -> builder
-                        .setMaxConnPerRoute(1)
-                        .setMaxConnTotal(1),
-                builder -> {});
+        final TestClient client = client();
+        final PoolingHttpClientConnectionManager connManager = client.getConnectionManager();
+        connManager.setMaxTotal(1);
+        connManager.setDefaultMaxPerRoute(1);
 
         final ScheduledExecutorService executorService = Executors.newSingleThreadScheduledExecutor();
         try {
